//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "Unit1.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"

String ArticlesXML;
String OldID;
String OldName;
float OldPrice;
int OldStock;

TForm1 *Form1;
//---------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent* Owner)
  : TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormCreate(TObject *Sender)
{
  panEdit->Hide();
  ArticlesXML = ExtractFilePath(Application->ExeName) + "Articles.xml";
  if (FileExists(ArticlesXML))
    cdsArt->LoadFromFile(ArticlesXML);
  else
    cdsArt->CreateDataSet();
  cdsArt->LogChanges = false;
  cdsArt->IndexFieldNames = "Name";  // order grid on field NAME
  cdsArt->First();
  ShowStatus("browsing", "OK");
  TFilterOptions FilterOptions;
  FilterOptions << foCaseInsensitive;
  cdsArt->FilterOptions = FilterOptions;
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormDestroy(TObject *Sender)
{
  cdsArt->MergeChangeLog();
  cdsArt->SaveToFile(ArticlesXML);
}
//---------------------------------------------------------------------------
void TForm1::ShowStatus(String DState, String Stat)
{ SBar->Panels->Items[0]->Text = "Records: " + IntToStr(cdsArt->RecordCount);
  SBar->Panels->Items[1]->Text = "State: " + DState;
  SBar->Panels->Items[2]->Text = "Status: " + Stat;
}
//----------------------------------------------------------------------------
void __fastcall TForm1::btnOKClick(TObject *Sender)
{
  String Err = "";
  bool ValidRecord = ValidID(Err);
  if (ValidRecord)
    ValidRecord = ValidData(Err);
  if (ValidRecord) {
    cdsArt->Post();
    if (cdsArt->State == dsInsert)
      ShowStatus("browsing", "Article was added");
    else
      ShowStatus("browsing", "Article was modified");
    UpdateUI(true);
  }
  else {
    if (cdsArt->State == dsInsert)
      ShowStatus("inserting", Err);
    else
      ShowStatus("editing", Err);
  }
}
//---------------------------------------------------------------------------
void __fastcall TForm1::btnCancelClick(TObject *Sender)
{
  ShowStatus("browsing", "nothing was changed");
  cdsArt->Cancel();
  UpdateUI(true);
}
//---------------------------------------------------------------------------
void __fastcall TForm1::cdsArtBeforeInsert(TDataSet *DataSet)
{
  ShowStatus("inserting", "");
  UpdateUI(false);
}
//---------------------------------------------------------------------------
void __fastcall TForm1::cdsArtBeforeEdit(TDataSet *DataSet)
{
  ShowStatus("editing", "");
  UpdateUI(false);
  OldID = cdsArt->FieldByName("ID")->AsString;
}
//---------------------------------------------------------------------------
bool TForm1::ValidID(String& Err)  // Err passed by REFERENCE
{
  bool Result = true;
  String ID = cdsArt->FieldByName("ID")->AsString;
  int LenID;
  Err = "";
  LenID = ID.Length();
  Result = (LenID == 4);
  if (! Result)
    Err = "Invalid ID, must be 4 characters ";
  else {
    cdsArtClone->CloneCursor(cdsArt, true, false);
    cdsArtClone->IndexFieldNames = "ID";
    // Note double brackets with ARRAYOFCONST
    if (cdsArtClone->FindKey(ARRAYOFCONST((ID)))) {
      if (cdsArt->State == dsInsert) {
        Result = false;
        Err = "Invalid, ID already exists";
      }
      else {         // editing
        if (cdsArt->FieldByName("ID")->AsString != OldID) {
          Result = false;
          Err = "Invalid, ID already exists";
        }
      }
    }
  }
  return Result;
}
//---------------------------------------------------------------------------
bool TForm1::ValidData(String& Err)  // Err passed by REFERENCE
{
  bool Result = true;
  Err = "";
  Result = (Trim(cdsArt->FieldByName("Name")->AsString) != "");
  if (! Result)
    Err = "Invalid, NAME can not be empty";
  else {
    Result = ((cdsArt->FieldByName("Price")->AsFloat) >= 0);
    if (! Result)
      Err = "Invalid, PRICE can not be negative";
    else {
     Result = ((cdsArt->FieldByName("Stock")->AsInteger) >= 0);
     if (! Result)
       Err = "Invalid, STOCK can not be negative";
    }
  }
  return Result;
}

void __fastcall TForm1::cdsArtAfterDelete(TDataSet *DataSet)
{
  ShowStatus("browsing", "OK");
}
//---------------------------------------------------------------------------
void __fastcall TForm1::btnFindIDClick(TObject *Sender)
{
  String ID = edFind->Text;
  ID = Trim(ID.UpperCase());
  edFind->Text = ID;
  cdsArt->IndexFieldNames = "ID";  // order grid on field ID
  if (cdsArt->FindKey(ARRAYOFCONST((ID))))
    ShowStatus("browsing", "ID was found");
  else
    ShowStatus("browsing", "Not found: ID " + ID);
}
//---------------------------------------------------------------------------
void __fastcall TForm1::btnFindNameClick(TObject *Sender)
{
  String Name = edFind->Text;
  Name = Trim(Name);
  cdsArt->IndexFieldNames = "Name";  // order grid on field Name
  cdsArt->FindNearest(ARRAYOFCONST((Name)));
  ShowStatus("browsing", "");
}
//---------------------------------------------------------------------------
void __fastcall TForm1::cbFilterClick(TObject *Sender)
{
  String Filt = Trim(edFind->Text);
  if (cbFilter->Checked) {
    cdsArt->Filter = "Name='" + Filt + "'";
    cdsArt->Filtered = true;
    ShowStatus("browsing", "filter ON");
  }
  else {
    cdsArt->Filtered = false;
    ShowStatus("browsing", "filter OFF");
  }
}
//---------------------------------------------------------------------------
void TForm1::UpdateUI(bool IsBrowsing)
{
  DBNavigator1->Enabled = IsBrowsing;
  DBGrid1->Enabled = IsBrowsing;
  panEdit->Visible = ! IsBrowsing;
  if (! IsBrowsing)
    edID->SetFocus();
  btnFindID->Enabled = IsBrowsing;
  btnFindName->Enabled = IsBrowsing;
  cbFilter->Enabled = IsBrowsing;
}
//---------------------------------------------------------------------------

